% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qualityIndices.R
\name{qualitySet}
\alias{qualitySet}
\title{Goodness of classifications for a set of k clusters.}
\usage{
qualitySet(data, k.set = c(2, 4), getImages = TRUE, seed = NULL)
}
\arguments{
\item{data}{A \code{\link{SummarizedExperiment}}.
The SummarizedExperiment must contain an assay with the following structure:
A valid header with names. The first  column of the header is the ID or name
of the instance of the dataset (e.g., ontology, pathway, etc.) on which the
metrics are measured.
The other columns of the header contains the names of the metrics.
The rows contains the measurements of the metrics for each instance in the dataset.}

\item{k.set}{A list of integer values of \code{k}, as in c(2,4,8).
The values must be contained in [2,15] range.}

\item{getImages}{Boolean. If true, a plot is displayed.}

\item{seed}{Positive integer. A seed for internal bootstrap.}
}
\value{
A list of \code{\link{SummarizedExperiment}} containing the silhouette width measurements and
cluster sizes from \code{k.set}.
}
\description{
The goodness of the classifications are assessed by validating the clusters
generated for a range of k values. For this purpose, we use the Silhouette width as validity index.
This index computes and compares the quality of the clustering outputs found
by the different metrics, thus enabling to measure the goodness of the
classification for both instances and metrics. More precisely, this measurement
provides an assessment of how similar an instance is to other instances from
the same cluster and dissimilar to the rest of clusters. The average on all
the instances quantifies how the instances appropriately are clustered. Kaufman
and Rousseeuw suggested the interpretation of the global Silhouette width score
as the effectiveness of the clustering structure. The values are in the
range [0,1], having the following meaning:

\itemize{
\item There is no substantial clustering structure: [-1, 0.25].
\item The clustering structure is weak and could be artificial: ]0.25, 0.50].
\item There is a reasonable clustering structure: ]0.50, 0.70].
\item A strong clustering structure has been found: ]0.70, 1].
}
}
\examples{
# Using example data from our package
data("rnaMetrics")
# Without plotting
dataFrameList = qualitySet(rnaMetrics, k.set=c(2,3), getImages = FALSE)

}
\references{
\insertRef{kaufman2009finding}{evaluomeR}
}
